"""This single file is needed to build the Client development dashboard."""

from promptmodel import DevApp
from main import client as main_client

# Example imports
# from <dirname> import < objectname>

app = DevApp()

# Example usage
# This is needed to integrate your codebase with the prompt engineering dashboard

app.include_client(main_client)

app.register_sample(
    name="function_call_test/1",
    content={"user_message": "What is the weather like in Boston?"},
)

app.register_sample(
    name="summarize/attention",
    content={
        "text": "The dominant sequence transduction models are based on complex recurrent or convolutional neural networks in an encoder-decoder configuration. The best performing models also connect the encoder and decoder through an attention mechanism. We propose a new simple network architecture, the Transformer, based solely on attention mechanisms, dispensing with recurrence and convolutions entirely. Experiments on two machine translation tasks show these models to be superior in quality while being more parallelizable and requiring significantly less time to train. Our model achieves 28.4 BLEU on the WMT 2014 English-to-German translation task, improving over the existing best results, including ensembles by over 2 BLEU. On the WMT 2014 English-to-French translation task, our model establishes a new single-model state-of-the-art BLEU score of 41.8 after training for 3.5 days on eight GPUs, a small fraction of the training costs of the best models from the literature. We show that the Transformer generalizes well to other tasks by applying it successfully to English constituency parsing both with large and limited training data. "
    },
)

from main import get_current_weather, get_current_weather_desc

app.register_function(get_current_weather_desc, get_current_weather)



# test.py
from promptmodel import PromptModel, DevClient, ChatModel
import promptmodel

promptmodel.init(use_cache=True)

client = DevClient()

prompt_model_config = PromptModel("summarize").get_config()

prompt_model_config = PromptModel("new_test").get_config()

chat_model_config = ChatModel("weather_bot").get_config()


from typing import Optional


def get_current_weather(location: str, unit: Optional[str] = None):
    return "13 degrees celsius"


get_current_weather_desc = {
    "name": "get_current_weather",
    "description": "Get the current weather in a given location",
    "parameters": {
        "type": "object",
        "properties": {
            "location": {
                "type": "string",
                "description": "The city and state, e.g. San Francisco, CA",
            },
            "unit": {"type": "string", "enum": ["celsius", "fahrenheit"]},
        },
        "required": ["location"],
    },
}

import json
import asyncio


@client.register
async def main():
    # res = await PromptModel("function_call_test").astream_and_parse(
    #     {"user_message": "Hello, how are you?"},
    #     functions=[get_current_weather_desc],
    # )
    # async for chunk in res:
    #     print(chunk.__dict__)
    # prompt = ChatModel("summary_bot").get_config()

    pass


# if __name__ == "__main__":
#     main()
#     async def sleep_func():
#         print(asyncio.all_tasks(loop=None))
#         asyncio.sleep(100)

#     asyncio.run(sleep_func())

# asyncio.run(main())



from promptmodel import init
from promptmodel import ChatModel
from promptmodel.types.response import PromptModelConfig, ChatModelConfig
from datetime import datetime

init(use_cache=True)

chat = ChatModel("weather_bot")
chat.add_messages([{"role": "user", "content": "hello"}])
res = chat.run()
print(res.raw_output)


# deployed_version_config: ChatModelConfig = ChatModel(
#     "weather_bot", version="deploy"
# ).get_config()

# print("DEPLOYED PROMPT")
# print(deployed_version_config.system_prompt)


# version_2_config: ChatModelConfig = ChatModel("weather_bot", version=2).get_config()

# print("VERSION 2 PROMPT")
# print(version_2_config.system_prompt)


# latest_version_config: ChatModelConfig = ChatModel(
#     "weather_bot", version="latest"
# ).get_config()

# print("LATEST VERSION PROMPT")
# print(latest_version_config.system_prompt)

session_uuid = chat.session_uuid
import asyncio

log_uuid_list = asyncio.run(
    chat.log(
        messages=[{"role": "user", "content": "hello"}],
        metadata={"timestamp": datetime.now().isoformat()},
    )
)
print(log_uuid_list)

asyncio.run(
    chat.log_score(
        score={"bleu": 1},
    )
)

asyncio.run(
    chat.log_score(
        log_uuid=log_uuid_list[0],
        score={"bleu": 1},
    )
)

log_uuid_list = asyncio.run(
    chat.log(
        openai_api_response=res.api_response,
        metadata={"timestamp": datetime.now().isoformat()},
    )
)
print(log_uuid_list)

asyncio.run(
    chat.log_score(
        log_uuid=log_uuid_list[0],
        score={"bleu": 1},
    )
)

log_uuid_list = asyncio.run(
    chat.log(
        openai_api_response=res.api_response,
        messages=[{"role": "user", "content": "hello"}],
        metadata={"timestamp": datetime.now().isoformat()},
    )
)



from typing import (
    List,
    Dict,
    Any,
    Optional,
)
from pydantic import BaseModel
from litellm.utils import (
    ModelResponse,
    Usage,
    Message,
    Choices,
    StreamingChoices,
    Delta,
    FunctionCall,
    Function,
    ChatCompletionMessageToolCall,
)
from openai._models import BaseModel as OpenAIObject
from openai.types.chat.chat_completion import *
from openai.types.chat.chat_completion_chunk import (
    ChoiceDeltaFunctionCall,
    ChoiceDeltaToolCall,
    ChoiceDeltaToolCallFunction,
)


class LLMResponse(OpenAIObject):
    pm_log_uuid: Optional[str] = None
    api_response: Optional[ModelResponse] = None
    raw_output: Optional[str] = None
    parsed_outputs: Optional[Dict[str, Any]] = None
    error: Optional[bool] = None
    error_log: Optional[str] = None
    function_call: Optional[FunctionCall] = None
    tool_calls: Optional[List[ChatCompletionMessageToolCall]] = None


class LLMStreamResponse(OpenAIObject):
    pm_log_uuid: Optional[str] = None
    api_response: Optional[ModelResponse] = None
    raw_output: Optional[str] = None
    parsed_outputs: Optional[Dict[str, Any]] = None
    error: Optional[bool] = None
    error_log: Optional[str] = None
    function_call: Optional[ChoiceDeltaFunctionCall] = None
    tool_calls: Optional[List[ChoiceDeltaToolCall]] = None


class FunctionModelConfig:
    """Response Class for FunctionModel.get_config()
    prompts: List[Dict[str, Any]] = []
        each prompt can have role, content, name, function_call, and tool_calls
    version_detail: Dict[str, Any] = {}
        version_detail has "model", "uuid", "parsing_type" and "output_keys".
    """

    def __init__(self, prompts, version_detail) -> None:
        self.prompts = prompts
        self.version_detail = version_detail


class PromptModelConfig(FunctionModelConfig):
    """Deprecated. Use FunctionModelConfig instead."""

    def __init__(self, prompts, version_detail) -> None:
        super().__init__(prompts, version_detail)


class ChatModelConfig:
    def __init__(
        self,
        system_prompt: str,
        version_detail: Dict,
        message_logs: Optional[Dict] = [],
    ) -> None:
        self.system_prompt = system_prompt
        self.version_detail = version_detail
        self.message_logs = message_logs


class FunctionSchema(BaseModel):
    """
    {
            "name": str,
            "description": Optional[str],
            "parameters": {
                "type": "object",
                "properties": {
                    "argument_name": {
                        "type": str,
                        "description": Optional[str],
                        "enum": Optional[List[str]]
                    },
                },
                "required": Optional[List[str]],
            },
        }
    """

    class _Parameters(BaseModel):
        class _Properties(BaseModel):
            type: str
            description: Optional[str] = ""
            enum: Optional[List[str]] = []

        type: str = "object"
        properties: Dict[str, _Properties] = {}
        required: Optional[List[str]] = []

    name: str
    description: Optional[str] = None
    parameters: _Parameters
